﻿namespace Assets
{
    using System;
    using System.Collections.Generic;
    using UnityEngine;

    /// <summary>
    /// Script for the mobile developer console mod.
    /// </summary>
    /// <seealso cref="UnityEngine.MonoBehaviour" />
    public class MobileDevConsoleScript : MonoBehaviour
    {
        /// <summary>
        /// The inputs processed
        /// </summary>
        private bool _inputsProcessed = false;

        /// <summary>
        /// Determines whether this instance is open.
        /// </summary>
        /// <returns>
        ///   <c>true</c> if this instance is open; otherwise, <c>false</c>.
        /// </returns>
        public bool IsOpen()
        {
            var devConsoleGameObject = UnityEngine.GameObject.Find("DeveloperConsole(Clone)");
            return devConsoleGameObject != null && devConsoleGameObject.activeSelf;
        }

        /// <summary>
        /// Executes the command.
        /// </summary>
        private void ExecuteCommand()
        {
            var devConsoleGameObject = UnityEngine.GameObject.Find("DeveloperConsole(Clone)");
            var devConsole = devConsoleGameObject.GetComponent("Jundroo.DevConsole.DeveloperConsole");
            Type devConsoleType = devConsole.GetType();
            var commandInputFieldObject = devConsoleType.GetField("CommandInputField").GetValue(devConsole);
            var commandInputFieldType = commandInputFieldObject.GetType();
            var commandText = commandInputFieldType.GetProperty("text").GetValue(commandInputFieldObject) as string;

            var commandObject = devConsoleType.GetField("_command", System.Reflection.BindingFlags.NonPublic | System.Reflection.BindingFlags.Instance).GetValue(devConsole);
            var commandType = commandObject.GetType();
            var parseAndUpdateMethod = commandType.GetMethod("ParseAndUpdate");
            parseAndUpdateMethod.Invoke(commandObject, new object[] { commandText });

            var executeMethod = commandType.GetMethod("Execute");
            executeMethod.Invoke(commandObject, null);

            commandInputFieldType.GetProperty("text").SetValue(commandInputFieldObject, string.Empty);
        }

        /// <summary>
        /// Unity OnGUI method.
        /// </summary>
        private void OnGUI()
        {
            if (this.IsOpen())
            {
                int padding = 20;
                int height = 100;
                int width = 250;
                if (GUI.Button(new Rect(padding, Screen.height - height - padding, width, height), "Execute Command"))
                {
                    this.ExecuteCommand();
                }

                if (GUI.Button(new Rect(width + padding + padding, Screen.height - height - padding, width, height), "Close Console"))
                {
                    this.ToggleConsole();
                }
            }
        }

        /// <summary>
        /// Toggles the console.
        /// </summary>
        private void ToggleConsole()
        {
            var go = GameObject.Find("SimplePlanesDevConsole");
            var console = go.GetComponent("Assets.Scripts.Startup.SimplePlanesDevConsoleScript");
            var consoleType = console.GetType();
            consoleType.GetMethod("ToggleConsole", System.Reflection.BindingFlags.NonPublic | System.Reflection.BindingFlags.Instance).Invoke(console, null);
        }

        /// <summary>
        /// Update is called once per frame
        /// </summary>
        private void Update()
        {
            if ((Input.touches.Length >= 3 || (Input.GetMouseButtonDown(1) && Application.platform == RuntimePlatform.WindowsPlayer)) && this._inputsProcessed == false)
            {
                this._inputsProcessed = true;
                this.ToggleConsole();
            }
            else if (Input.touches.Length == 0)
            {
                this._inputsProcessed = false;
            }
        }
    }
}